#include <stdio.h>
#include <fstream>
#include <iostream>
#include <direct.h>
#include "paramSimul.h"
#include "const.h"


using namespace std;

bool Open_configure_file(fstream &file, char *name);
double Get_parameter(fstream &file, const char *name, double _default);
char* Get_character(fstream &file, const char *name, const char* _default);
void eatline(fstream &file);
void Decide_test_environment(char * test_environment_char);
void Decide_pathloss_model(char * pathloss_model_char);
void make_directory(void);


void Set_simul_param(int argc, char *argv[])
{
	calibration = false;

	cout << "-------Simulation_parametors--------" << endl;
	if (argc == 2)
	{
		fstream infile;
		if (!Open_configure_file(infile, argv[1]))
		{
			cout << "error : Can't find the file " << "'" << argv[1] << "'" << endl;
			exit(1);
		}
		
		_seed = int(Get_parameter(infile, "_seed", 100000));
		num_drops = int(Get_parameter(infile, "num_drops", 10000));
		run_times = int(Get_parameter(infile, "run_times", 1000));
		num_user_per_sector = int(Get_parameter(infile, "num_user_per_sector", 10));
		num_rbs_in_1ms = int(Get_parameter(infile, "num_rbs_in_1ms", 50));
		bs_antenna_array_type = int(Get_parameter(infile, "bs_antenna_array_type", 1));
		
		P_bs = int(Get_parameter(infile, "P_bs", 1));
		Mg_bs = int(Get_parameter(infile, "Mg_bs", 1));
		Ng_bs = int(Get_parameter(infile, "Ng_bs", 1));
		M_bs = int(Get_parameter(infile, "M_bs", 1));
		N_bs = int(Get_parameter(infile, "N_bs", 1));
		dg_H_bs = double(Get_parameter(infile, "dg_H_bs", 1));
		dg_V_bs = double(Get_parameter(infile, "dg_V_bs", 1));
		d_H_bs = double(Get_parameter(infile, "d_H_bs", 1));
		d_V_bs = double(Get_parameter(infile, "d_V_bs", 1));
	

		ue_antenna_array_type = int(Get_parameter(infile, "bs_antenna_array_type", 1));
		
		P_ue = int(Get_parameter(infile, "P_ue", 1));
		Mg_ue = int(Get_parameter(infile, "Mg_ue", 1));
		Ng_ue = int(Get_parameter(infile, "Ng_ue", 1));
		M_ue = int(Get_parameter(infile, "M_ue", 1));
		N_ue = int(Get_parameter(infile, "N_ue", 1));
		dg_H_ue = double(Get_parameter(infile, "dg_H_ue", 0));
		dg_V_ue = double(Get_parameter(infile, "dg_V_ue", 0));
		d_H_ue = double(Get_parameter(infile, "d_H_ue", 0));
		d_V_ue = double(Get_parameter(infile, "d_V_ue", 0));


		num_rx_antenna_element = P_bs * Mg_bs * Ng_bs * M_bs * N_bs;
		num_tx_antenna_element = P_ue * Mg_ue * Ng_ue * M_ue * N_ue;
		num_compute_coef = int(Get_parameter(infile, "num_compute_coef", 1));
		
		num_neighbor = int(Get_parameter(infile, "num_neighbor", 4));
		grid_interval = int(Get_parameter(infile, "grid_interval", 10));

		char * test_environment_char = Get_character(infile, "test_environment", "Dense_Urban_eMBB_A");
		char * pathloss_char = Get_character(infile, "pathloss_model", "UMa_A");
				
		
		Decide_test_environment(test_environment_char);
		Decide_pathloss_model(pathloss_char);
		file_name = Get_character(infile, "file_name", "output.txt");	

	}
	
	else if (argc == 1)
	{
		_seed = 1000;
		num_drops = 100;
		run_times = 1000;
		num_user_per_sector = 10;
		num_rbs_in_1ms = 50;
		bs_antenna_array_type = 0;

		P_bs = 1;
		Mg_bs = 1;
		Ng_bs = 1;
		dg_H_bs = 0;
		dg_V_bs = 0;
		M_bs = 1;
		N_bs = 1;

		ue_antenna_array_type = 0;

		P_ue = 1;
		Mg_ue = 1;
		Ng_ue = 1;
		dg_H_ue =0;
		dg_V_ue = 0;
		M_ue = 1;
		N_ue = 1;

		
		
		
		num_rx_antenna_element = 1;
		num_tx_antenna_element = 1;
		num_compute_coef = 1;
		




		num_neighbor = 4;
		grid_interval = 10;
		
		test_environment = Dense_Urban_eMBB_A;
		pathloss_model = UMa_A;
		

		file_name = (char *)"output.txt";

	}
	else if (argc > 2)
	{
		cout << " Usage : run.exe filename " << endl;
		exit(1);
	}

	

	if (test_environment <3 ) // IMT2020 eval. 8.3.1
	{
		num_bs = 12;
		num_sector = 12;
	}
	else if (test_environment == Dense_Urban_eMBB_C_Micro)
	{
		num_bs = 19 * 3 * 3;
		num_sector = 19 * 3 * 3;
	}
	else
	{
		num_bs = 19;
		num_sector = 19 * 3;
	}

	
	num_ue = num_user_per_sector * num_sector;

	make_directory();


	cout << "num_drops           : " << num_drops << endl;
	cout << "run_times           : " << run_times << endl;
	cout << "num_user_per_sector : " << num_user_per_sector << endl;
	cout << "num_rx_antenna_element      : " << num_rx_antenna_element << endl;
	cout << "num_tx_antenna_element      : " << num_tx_antenna_element << endl;
	cout << "num_sector         : " << num_sector << endl;
	cout << "grid_interval        : " << grid_interval << endl;
	switch (test_environment)
	{
	case InH_eMBB_A :
		cout << "test_environment    : InH_eMBB_A" << endl;
		break;
	case InH_eMBB_B :
		cout << "test_environment    : InH_eMBB_B" << endl;
		break;
	case InH_eMBB_C :
		cout << "test_environment    : InH_eMBB_C" << endl;
		break;
	case Dense_Urban_eMBB_A :
		cout << "test_environment    : Dense_Urban_eMBB_A" << endl;
		break;
	case Dense_Urban_eMBB_B :
		cout << "test_environment    : Dense_Urban_eMBB_B" << endl;
		break;
	case Dense_Urban_eMBB_C_Macro :
		cout << "test_environment    : Dense_Urban_eMBB_C_Macro" << endl;
		break;
	case Dense_Urban_eMBB_C_Micro :
		cout << "test_environment    : Dense_Urban_eMBB_C_Micro" << endl;
		break;
	case Rural_eMBB_A :
		cout << "test_environment    : Rural_eMBB_A" << endl;
		break;
	case Rural_eMBB_B :
		cout << "test_environment    : Rural_eMBB_B" << endl;
		break;
	case Rural_eMBB_C :
		cout << "test_environment    : Rural_eMBB_C" << endl;
		break;
	default:
		cout << "check test_environment input file" << endl;
		break;
	}

	switch (pathloss_model)
	{
	case InH_A:
		cout << "pathloss_model      : InH_A" << endl;
		break;
	case InH_B:
		cout << "pathloss_model      : InH_B" << endl;
		break;
	case UMa_A:
		cout << "pathloss_model      : UMa_A" << endl;
		break;
	case UMa_B:
		cout << "pathloss_model      : UMa_B" << endl;
		break;
	case UMi_A:
		cout << "pathloss_model      : UMi_A" << endl;
		break;
	case UMi_B:
		cout << "pathloss_model      : UMi_B" << endl;
		break;
	case RMa_A:
		cout << "pathloss_model      : RMa_A" << endl;
		break;
	case RMa_B:
		cout << "pathloss_model      : RMa_B" << endl;
		break;
	default:
		cout << "check pathloss_model input file" << endl;
		break;
	}
	
}

bool Open_configure_file(fstream &file, char *name)
{
	bool status;

	file.open(name, ios::in);
	if (file.fail())
		status = false;
	else
		status = true;
	return status;
}


double Get_parameter(fstream &file, const char *name, double _default)
{

	double argument = 0.0;
	bool I_got_it = false;
	char temp[30];
	while (!file.eof() && !I_got_it )
	{

		char comment_notice;
		file >> comment_notice;
		if (comment_notice == '#')
			eatline(file);

		file.seekg(-1, ios::cur);
		file >> temp;
		
		if (!strcmp(name, temp))
		{
			I_got_it = true;
			file >> argument;
		}
	}
	
	file.clear();
	file.seekg(0, ios::beg);

	if (I_got_it)
		return(argument);
	else
		return(_default);
}

char* Get_character(fstream &file, const char *name, const char* _default)
{
	char* argument = new char[80];
	bool I_got_it = false;
	char temp[80];

	while (!file.eof() && !I_got_it)
	{
		char comment_notice;
		file >> comment_notice;
		if (comment_notice == '#')
			eatline(file);

		file.seekg(-1, ios::cur);
		file >> temp;
		if (!strcmp(name, temp))
		{
			I_got_it = true;
			file >> argument;
		}
	}
	file.clear();
	file.seekg(0, ios::beg);

	if (I_got_it)
		return(argument);
	else
		return((char*)_default);
}

void eatline(fstream &file)
{
	char null_input[1000];
	file.getline(null_input, 1000);
}


void Decide_test_environment(char * test_environment_char)
{
	char *test_environment_type_[10];

	test_environment_type_[0] = (char *)"InH_eMBB_A";
	test_environment_type_[1] = (char *)"InH_eMBB_B";
	test_environment_type_[2] = (char *)"InH_eMBB_C";
	test_environment_type_[3] = (char *)"Dense_Urban_eMBB_A";
	test_environment_type_[4] = (char *)"Dense_Urban_eMBB_B";
	test_environment_type_[5] = (char *)"Dense_Urban_eMBB_C_Macro";
	test_environment_type_[6] = (char *)"Dense_Urban_eMBB_C_Micro";
	test_environment_type_[7] = (char *)"Rural_eMBB_A";
	test_environment_type_[8] = (char *)"Rural_eMBB_B";
	test_environment_type_[9] = (char *)"Rural_eMBB_C";

	for (int idx = 0; idx < 10; idx++)
	{

		if (!strcmp(test_environment_char, test_environment_type_[idx]))
		{
			test_environment = idx;
		}
	}
}

void Decide_pathloss_model(char * pathloss_model_char)
{
	char *pathloss_type_[8];

	pathloss_type_[0] = (char *) "InH_A";
	pathloss_type_[1] = (char *) "InH_B";
	pathloss_type_[2] = (char *) "UMa_A";
	pathloss_type_[3] = (char *) "UMa_B";
	pathloss_type_[4] = (char *) "UMi_A";
	pathloss_type_[5] = (char *) "UMi_B";
	pathloss_type_[6] = (char *) "RMa_A";
	pathloss_type_[7] = (char *) "RMa_B";

	for (int idx = 0; idx < 8; idx++)
	{

		if (!strcmp(pathloss_model_char, pathloss_type_[idx]))
		{
			pathloss_model = idx;
		}
	}
}


void make_directory(void)
{
	char forder_name[50];

	for (int idx = 0; idx < 2 * sizeof(file_name); idx++)
	{
		if (file_name[idx] == '.')
		{

			if (file_name[idx + 1] == '/')
			{
				int k = 1;
				while (file_name[idx + 1 + k] != '/')
				{
					forder_name[k - 1] = file_name[idx + 1 + k];
					k++;
				}
				forder_name[k - 1] = '\0';
			}
		}
	}
	_mkdir(forder_name);
	path[0] = '.';   path[1] = '/';   path[2] = '\0';
	strcat_s(path,sizeof(path), forder_name);
}
